@import "tailwindcss";

@theme {
  --font-sans:
    "Space Grotesk", "IBM Plex Sans", ui-sans-serif, system-ui, sans-serif;
  --font-mono: "JetBrains Mono", ui-monospace, "Cascadia Code", monospace;

  --color-accent: #22d3ee;
  --color-accent-dim: #0891b2;

  --color-score-hot: #4ade80;
  --color-score-hot-glow: #22c55e40;
  --color-score-high: #22c55e;
  --color-score-mid: #eab308;
  --color-score-low: #71717a;
  --color-score-noise: #3f3f46;

  --color-tier-high-signal: #4ade80;
  --color-tier-market-relevant: #22d3ee;
  --color-tier-news: #a78bfa;
  --color-tier-noise: #52525b;
}

@layer base {
  * {
    @apply border-zinc-800;
  }

  body {
    @apply bg-zinc-950 text-zinc-50;
    background-image:
      radial-gradient(
        1200px 550px at 50% -100px,
        rgba(34, 211, 238, 0.08),
        transparent 60%
      ),
      linear-gradient(180deg, #080a10 0%, #090b12 55%, #090c12 100%);
    font-feature-settings: "cv02", "cv03", "cv04", "cv11";
  }

  /* Scrollbar styling */
  ::-webkit-scrollbar {
    width: 6px;
    height: 6px;
  }
  ::-webkit-scrollbar-track {
    background: transparent;
  }
  ::-webkit-scrollbar-thumb {
    background: #3f3f46;
    border-radius: 3px;
  }
  ::-webkit-scrollbar-thumb:hover {
    background: #52525b;
  }

  /* Always show actions on touch devices (no hover) */
  @media (hover: none) {
    .tweet-actions {
      opacity: 1;
    }
  }
}
